//  fragment shader for materials with 3d noise

// uniforms - note they must be used if they are to be accessed in the calling program

uniform int   Clip1;
uniform vec3  MaterialC;              // Material second colour
uniform float SpecularPower;          // specularity  (range 10 - 80 )
uniform float SpecularContribution;   // mix of specular contribution - turns on/off spec 0.0/1.0
uniform int   ShaderID;               // shader ID
uniform int   ShaderInternal;         // TRUE (1) if Internal Shader
uniform float ShaderParameter;        // Shader Parameter
uniform float MixRatio;               // Reflection Fraction
uniform vec3  ScalingV;

uniform vec3  C1;
uniform float Sharpness;
uniform float Coverage;

uniform vec3 AntiNormal1;        // Transform matrix for normal to undo camera
uniform vec3 AntiNormal2;        // Connection to a mat3 uniform  - does not work for some reason
uniform vec3 AntiNormal3;

uniform sampler3D NoiseMap;           // Map for 3D noise
uniform sampler3D RefMap3D;           
//uniform sampler2D RefMap;             // Reflection sampler
//uniform sampler2D NormalMap;          // Map for bump mapping

uniform sampler2D ShaderTexture;

const float DiffuseContribution=1.0;
const vec3  Xunitvec = vec3 (1.0, 0.0, 0.0);
const vec3  Yunitvec = vec3 (0.0, 1.0, 0.0);


varying vec3 tnorm;       // interpolated surface normal
varying vec4 PP;          // vertex global coord
varying vec3 texpos;      // shader coord relative to material shader axis

varying vec3 wvec;
varying vec3 uvec;
varying vec3 vvec;

void CalcLight(in vec3, in vec3, in vec3, inout vec4, inout vec4); // Light function (External)

float triangle(in float value){
  float offset;
  if (value >= 0.0) offset = value - floor(value);
  else offset = value - (-1.0 - floor(abs(value)));
  if (offset >= 0.5) return (2.0 * (1.0 - offset));
  else return (2.0 * offset);
}

void main(void){
    if(Clip1 == 1){  // Use this as alternative to use the buid in clipping to make ATI/nVidia Compatible
      float d=dot(PP.xyz,gl_ClipPlane[0].xyz) + gl_ClipPlane[0].w;
      if( d < 0.0)discard;
    }
    // compute the fragment position in eye coordinates
    vec3 ecPosition = vec3(PP);
    // compute a unit vector in direction of viewing position
    vec3 viewVec    = normalize(-ecPosition);
    // must re-normalize interpolated vector to get Phong shading
    vec3 vnorm      = normalize(tnorm);
 
    // Prepare for reflection using pseudo environment map
    vec3  reflectDir = reflect(ecPosition,vnorm);
    mat3 AntiNormal; 
    AntiNormal[0]=AntiNormal1;  // Must do assignment in this way on ATI hardware
    AntiNormal[1]=AntiNormal2;
    AntiNormal[2]=AntiNormal3;
    reflectDir=AntiNormal*reflectDir;
    vec2 index;
    index.y = dot(normalize(reflectDir), Yunitvec);
    reflectDir.y = 0.0;
    index.x = dot(normalize(reflectDir), Xunitvec) * 0.5;
    if (reflectDir.z >= 0.0)index = (index + 1.0) * 0.5;
    else {
        index.t = (index.t + 1.0) * 0.5;
        index.s = (-index.s) * 0.5 + 1.0;
     }
//    vec3 envColor = vec3 (texture2D(RefMap, index)); 
    vec3 envColor = vec3 (texture3D(RefMap3D, vec3(index,0.5))); 
    // Calculate the Phong lighting
    vec4 diffuse=vec4(0.0,0.0,0.0,1.0);
    vec4 spec=vec4(0.0,0.0,0.0,0.0);
    CalcLight(ecPosition,viewVec,vnorm,diffuse,spec);

    // calcuate any surface colour - using any algorithm
    vec4 surface_colour=gl_Color;

    vec4 Ns;
    vec3 texpos1 = texpos*ScalingV;
    Ns=texture3D(NoiseMap,texpos1);
    float noiseH=(Ns[0]*2.0);
    noiseH = triangle(noiseH);
    noiseH = (0.3+0.9*noiseH);
    if(noiseH > Coverage){
      noiseH=(noiseH-Coverage)/(1.2-Coverage);
      noiseH=noiseH*(1.0+1.0*Sharpness);
      noiseH=min(noiseH,1.0);
      surface_colour=mix(vec4(C1,1.0),gl_Color,(1.0-noiseH));
    }

    surface_colour = mix(surface_colour,vec4(texture2D(ShaderTexture, texpos.xy).rgb,1.0),0.5); 

    // now make the fragment
    gl_FragColor = mix(surface_colour * DiffuseContribution * diffuse, vec4(envColor,1.0),MixRatio)
                 + spec * SpecularContribution;
}
