// fragment shader for plugin textures

// uniforms - note they must be used if they are to be accessed in the calling program

uniform int   Clip1;
uniform vec3  MaterialC;              // Material second colour
uniform vec3  ScalingV;
uniform float SpecularPower;          // specularity  (range 10 - 80 )
uniform float SpecularContribution;   // mix of specular contribution - turns on/off spec 0.0/1.0
uniform float MixRatio;               // Reflection Fraction

uniform vec3 ColourC1;
uniform vec3 ColourC2;
uniform vec3 ColourC3;
uniform float Size;
uniform float Edge;
 

uniform vec3 AntiNormal1;        // Transform matrix for normal to undo camera
uniform vec3 AntiNormal2;        // Connection to a mat3 uniform  - does not work for some reason
uniform vec3 AntiNormal3;

uniform sampler2D RefMap;             // Reflection sampler

const float DiffuseContribution=1.0;
const vec3  Xunitvec = vec3 (1.0, 0.0, 0.0);
const vec3  Yunitvec = vec3 (0.0, 1.0, 0.0);

varying vec3 tnorm;       // interpolated surface normal
varying vec4 PP;          // vertex global coord
varying vec3 texpos;      // shader coord relative to material shader axis

varying vec3 wvec;
varying vec3 uvec;
varying vec3 vvec;

void CalcLight(in vec3, in vec3, in vec3, inout vec4, inout vec4); // Light function (External)

vec3 ppp[6];

void SetUpPPP(void){
  ppp[0]=vec3(0.0,0.5,0.5);
  ppp[1]=vec3(0.5,1.0,0.5);
  ppp[2]=vec3(1.0,0.5,0.5);
  ppp[3]=vec3(0.5,0.0,0.5);
  ppp[4]=vec3(0.5,0.5,0.0);
  ppp[5]=vec3(0.5,0.5,1.0);
}

void main(void){
    SetUpPPP();
    if(Clip1 == 1){  // Use this as alternative to use the buid in clipping to make ATI/nVidia Compatible
      float d=dot(PP.xyz,gl_ClipPlane[0].xyz) + gl_ClipPlane[0].w;
      if( d < 0.0)discard;
    }
    // compute the fragment position in eye coordinates
    vec3 ecPosition = vec3(PP);
    // compute a unit vector in direction of viewing position
    vec3 viewVec    = normalize(-ecPosition);
    // must re-normalize interpolated vector to get Phong shading
    vec3 vnorm      = normalize(tnorm);
 
    vec3  reflectDir = reflect(ecPosition,vnorm);
    mat3 AntiNormal; 
    AntiNormal[0]=AntiNormal1;  
    AntiNormal[1]=AntiNormal2;
    AntiNormal[2]=AntiNormal3;
    reflectDir=AntiNormal*reflectDir;
    vec2 index;
    index.y = dot(normalize(reflectDir), Yunitvec);
    reflectDir.y = 0.0;
    index.x = dot(normalize(reflectDir), Xunitvec) * 0.5;
    if (reflectDir.z >= 0.0)index = (index + 1.0) * 0.5;
    else {
        index.t = (index.t + 1.0) * 0.5;
        index.s = (-index.s) * 0.5 + 1.0;
    }
    vec3 envColor = vec3 (texture2D(RefMap, index)); 
    int i;
    vec3 p;
    p = texpos*ScalingV;
    p=fract(p);
    vec4 surface_colour=gl_Color;
    float a,zb=0.0;
    float e=(Edge/100.0)*Size;
    for(i=0; i<6 ;i++){
      if((a=length(p-ppp[i])) < Size){
        if((Size-a) > zb){
          zb=(Size-a);
          if(i == 0 || i == 2){
            surface_colour=vec4(ColourC1,1.0);
          }
          else if(i == 1 || i == 3){
            surface_colour=vec4(ColourC2,1.0);
          }
          else{
            surface_colour=vec4(ColourC3,1.0);
          }
          if(a > e){
            surface_colour=mix(surface_colour,gl_Color,(a-e)/(Size-e));
          }
        }
      }
    }
    vec4 diffuse=vec4(0.0,0.0,0.0,1.0);
    vec4 spec=vec4(0.0,0.0,0.0,0.0);
    CalcLight(ecPosition,viewVec,vnorm,diffuse,spec);
    gl_FragColor = mix(surface_colour * DiffuseContribution * diffuse, vec4(envColor,1.0),MixRatio)
                 + spec * SpecularContribution;
}
