// fragment shader for plugin textures

// uniforms - note they must be used if they are to be accessed in the calling program

uniform int   Clip1;
uniform vec3  MaterialC;              // Material second colour
uniform float SpecularPower;          // specularity  (range 10 - 80 )
uniform float SpecularContribution;   // mix of specular contribution - turns on/off spec 0.0/1.0
uniform int   ShaderID;               // shader ID
uniform int   ShaderInternal;         // TRUE (1) if Internal Shader
uniform float ShaderParameter;        // Shader Parameter
uniform float MixRatio;               // Reflection Fraction

uniform vec3 AntiNormal1;        // Transform matrix for normal to undo camera
uniform vec3 AntiNormal2;        // Connection to a mat3 uniform  - does not work for some reason
uniform vec3 AntiNormal3;

uniform sampler3D NoiseMap;           // Map for 3D noise
uniform sampler2D RefMap;             // Reflection sampler
uniform sampler2D NormalMap;          // Map for bump mapping

const float DiffuseContribution=1.0;
const vec3  Xunitvec = vec3 (1.0, 0.0, 0.0);
const vec3  Yunitvec = vec3 (0.0, 1.0, 0.0);


varying vec3 tnorm;       // interpolated surface normal
varying vec4 PP;          // vertex global coord
varying vec3 texpos;      // shader coord relative to material shader axis

varying vec3 wvec;
varying vec3 uvec;
varying vec3 vvec;

void CalcLight(in vec3, in vec3, in vec3, inout vec4, inout vec4); // Light function (External)

#define DD 0.01  


void main(void){
    if(Clip1 == 1){  // Use this as alternative to use the buid in clipping to make ATI/nVidia Compatible
      float d=dot(PP.xyz,gl_ClipPlane[0].xyz) + gl_ClipPlane[0].w;
      if( d < 0.0)discard;
    }
    // compute the fragment position in eye coordinates
    vec3 ecPosition = vec3(PP);
    // compute a unit vector in direction of viewing position
    vec3 viewVec    = normalize(-ecPosition);
    // must re-normalize interpolated vector to get Phong shading
    vec3 vnorm      = normalize(tnorm);
 
    // Prepare for reflection using pseudo environment map
    vec3  reflectDir = reflect(ecPosition,vnorm);
    mat3 AntiNormal; 
    AntiNormal[0]=AntiNormal1;  // Must do assignment in this way on ATI hardware
    AntiNormal[1]=AntiNormal2;
    AntiNormal[2]=AntiNormal3;
    reflectDir=AntiNormal*reflectDir;
    vec2 index;
    index.y = dot(normalize(reflectDir), Yunitvec);
    reflectDir.y = 0.0;
    index.x = dot(normalize(reflectDir), Xunitvec) * 0.5;
    if (reflectDir.z >= 0.0)index = (index + 1.0) * 0.5;
    else {
        index.t = (index.t + 1.0) * 0.5;
        index.s = (-index.s) * 0.5 + 1.0;
     }
    vec3 envColor = vec3 (texture2D(RefMap, index)); 

    if(ShaderID == 1){ // do bump mapping using noise
      vec3 left=texpos;   left.x  += 0.1;
      vec3 right=texpos;  right.y += 0.1;
      vec3 top=texpos;    top.z   += 0.1;
      vec3  delta0 = vec3(texture3D(NoiseMap,texpos.xyz)); 
      float delta1 = texture3D(NoiseMap,left.xyz).x; 
      float delta2 = texture3D(NoiseMap,right.xyz).y; 
      float delta3 = texture3D(NoiseMap,top.xyz).z; 
      delta1=(delta1-delta0.x)*ShaderParameter;
      delta2=(delta2-delta0.y)*ShaderParameter;
      delta3=(delta3-delta0.z)*ShaderParameter;
      vnorm=normalize(vnorm + vec3(8.0,0.0,0.0)*delta1 + vec3(0.0,8.0,0.0)*delta2 + vec3(0.0,0.0,8.0)*delta3);
    }
    else if(ShaderID == 2){ // do bump mapping using normal map
      float sfactor=15.0;
      float nw=abs(dot(vnorm,wvec));
      float nu=abs(dot(vnorm,uvec));
      float nv=abs(dot(vnorm,vvec));
      vec2  c=texpos.xy;              
      vec2 r=vec2(c.x+DD,c.y);
      vec2 t=vec2(c.x,c.y+DD);
      float cval  = texture2D(NormalMap,c).r; 
      float grada = cval - (texture2D(NormalMap,r).r); 
      float gradb = cval - (texture2D(NormalMap,t).r);
      vec3 nDelta = nw*(grada*uvec + gradb*vvec);
      c=texpos.yz;  c.y *= -1.0; 
      r=vec2(c.x+DD,c.y);
      t=vec2(c.x,c.y+DD);
      cval  = texture2D(NormalMap,c).g; 
      grada = cval - (texture2D(NormalMap,r).g); 
      gradb = cval - (texture2D(NormalMap,t).g);
      nDelta = nDelta + nu*(grada*vvec + gradb*wvec);
      c=texpos.xz; c.y *= -1.0;
      r=vec2(c.x+DD,c.y);
      t=vec2(c.x,c.y+DD);
      cval  = texture2D(NormalMap,c).b; 
      grada = cval - (texture2D(NormalMap,r).b); 
      gradb = cval - (texture2D(NormalMap,t).b);
      nDelta = nDelta + nv*(grada*wvec + gradb*uvec);
      vnorm += nDelta*sfactor*ShaderParameter;  // different scale can be used to account for different map resolution
      vnorm=normalize(vnorm);
    }

    // Calculate the Phong lighting
    vec4 diffuse=vec4(0.0,0.0,0.0,1.0);
    vec4 spec=vec4(0.0,0.0,0.0,0.0);
    CalcLight(ecPosition,viewVec,vnorm,diffuse,spec);

    // apply the algorithm surface colour - using any algorithm
    vec4 surface_colour;

surface_colour=vec4(1.0,1.0,0.0,1.0);
diffuse=vec4(1.0,1.0,1.0,1.0);
    // now make the fragment
    gl_FragColor = mix(surface_colour * DiffuseContribution * diffuse, vec4(envColor,1.0),MixRatio)
                 + spec * SpecularContribution;
}
