// carving - fragment shader for materials with 3d noise

// uniforms - note they must be used if they are to be accessed in the calling program

uniform int   Clip1;
uniform vec3  MaterialC;              // Material second colour
uniform float SpecularPower;          // specularity  (range 10 - 80 )
uniform float SpecularContribution;   // mix of specular contribution - turns on/off spec 0.0/1.0
uniform int   ShaderID;               // shader ID
uniform int   ShaderInternal;         // TRUE (1) if Internal Shader
uniform float ShaderParameter;        // Shader Parameter
uniform float MixRatio;               // Reflection Fraction

uniform vec3 ScalingV;

uniform vec3 AntiNormal1;        // Transform matrix for normal to undo camera
uniform vec3 AntiNormal2;        // Connection to a mat3 uniform  - does not work for some reason
uniform vec3 AntiNormal3;

uniform sampler3D NoiseMap;           // Map for 3D noise
uniform sampler3D RefMap3D;           
//uniform sampler2D RefMap;             // Reflection sampler
//uniform sampler2D NormalMap;          // Map for bump mapping

const float DiffuseContribution=1.0;
const vec3  Xunitvec = vec3 (1.0, 0.0, 0.0);
const vec3  Yunitvec = vec3 (0.0, 1.0, 0.0);

const float t=0.4,dt=0.18;

varying vec3 tnorm;       // interpolated surface normal
varying vec4 PP;          // vertex global coord
varying vec3 texpos;      // shader coord relative to material shader axis

varying vec3 wvec;
varying vec3 uvec;
varying vec3 vvec;

void CalcLight(in vec3, in vec3, in vec3, inout vec4, inout vec4); // Light function (External)

float Noise3(in vec3 p){
  float noisef=texture3D(NoiseMap,p)[0]*3.0;
  return 0.5*(1+sin(6.0*3.1415926*noisef));
}

void main(void){
    if(Clip1 == 1){  // Use this as alternative to use the buid in clipping to make ATI/nVidia Compatible
      float d=dot(PP.xyz,gl_ClipPlane[0].xyz) + gl_ClipPlane[0].w;
      if( d < 0.0)discard;
    }
    // compute the fragment position in eye coordinates
    vec3 ecPosition = vec3(PP);
    // compute a unit vector in direction of viewing position
    vec3 viewVec    = normalize(-ecPosition);
    // must re-normalize interpolated vector to get Phong shading
    vec3 vnorm      = normalize(tnorm);
 
    // Prepare for reflection using pseudo environment map
    vec3  reflectDir = reflect(ecPosition,vnorm);
    mat3 AntiNormal; 
    AntiNormal[0]=AntiNormal1;  // Must do assignment in this way on ATI hardware
    AntiNormal[1]=AntiNormal2;
    AntiNormal[2]=AntiNormal3;
    reflectDir=AntiNormal*reflectDir;
    vec2 index;
    index.y = dot(normalize(reflectDir), Yunitvec);
    reflectDir.y = 0.0;
    index.x = dot(normalize(reflectDir), Xunitvec) * 0.5;
    if (reflectDir.z >= 0.0)index = (index + 1.0) * 0.5;
    else {
        index.t = (index.t + 1.0) * 0.5;
        index.s = (-index.s) * 0.5 + 1.0;
     }
//    vec3 envColor = vec3 (texture2D(RefMap, index)); 
    vec3 envColor = vec3 (texture3D(RefMap3D, vec3(index,0.5))); 

    // calcuate any surface colour - using any algorithm
    vec4 surface_colour=gl_Color;
    
    vec3 pp=texpos*ScalingV*2.0;
    float nois=Noise3(pp);
    if(nois > t && nois < t+3.0*dt){
      float xr=Noise3(pp+vec3(0.01,0.0,0.0));
      float xl=Noise3(pp-vec3(0.01,0.0,0.0));
      float yr=Noise3(pp+vec3(0.0,0.01,0.0));
      float yl=Noise3(pp-vec3(0.0,0.01,0.0));
      float zr=Noise3(pp+vec3(0.0,0.0,0.01));
      float zl=Noise3(pp-vec3(0.0,0.0,0.01));
      vec3 v;
      v[0]= -(xr-xl); v[1]= -(yr-yl); v[2]= -(zr-zl);
      normalize(v);
      vec3 vt;
      vt=v[0]*uvec + v[1]*vvec + v[2]*wvec;
      float s;
      if(nois < t+dt){
        s=(nois-t)/dt;
        s = -s*s;
      }
      else if(nois < t+2.0*dt)s=0.0;
      else{
        s=1.0-(nois-(t+2.0*dt))/dt;
        s = s*s;
      }
      vnorm += s*vt;
      normalize(vnorm);
    }
    // Calculate the Phong lighting
    vec4 diffuse=vec4(0.0,0.0,0.0,1.0);
    vec4 spec=vec4(0.0,0.0,0.0,0.0);
    CalcLight(ecPosition,viewVec,vnorm,diffuse,spec);
 
    // now make the fragment
    gl_FragColor = mix(surface_colour * DiffuseContribution * diffuse, vec4(envColor,1.0),MixRatio)
                 + spec * SpecularContribution;
}
